@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 # vi:ts=4:et
 
+from . import localhost
 import pycurl
 import unittest
 import nose.tools
@@ -29,7 +30,7 @@ class SetoptUnicodeTest(unittest.TestCase):
         self.check(util.u('p=Москва').encode('utf8'), util.u('Москва'))
 
     def check(self, send, expected):
-        self.curl.setopt(pycurl.URL, 'http://localhost:8380/param_utf8_hack')
+        self.curl.setopt(pycurl.URL, 'http://%s:8380/param_utf8_hack' % localhost)
         sio = util.BytesIO()
         self.curl.setopt(pycurl.WRITEFUNCTION, sio.write)
         self.curl.setopt(pycurl.POSTFIELDS, send)
