@@ -2,15 +2,18 @@
 # -*- coding: utf-8 -*-
 # vi:ts=4:et
 
+from . import util
 import setup as pycurl_setup
 import unittest
 import os, os.path, sys
 import nose.plugins.skip
-
+import functools
 try:
-    import functools
+    # Python 2
+    from StringIO import StringIO
 except ImportError:
-    import functools_backport as functools
+    # Python 3
+    from io import StringIO
 
 def set_env(key, new_value):
     old_value = os.environ.get(key)
@@ -58,17 +61,21 @@ def min_python_version(*spec):
     return decorator
 
 class SetupTest(unittest.TestCase):
+
+    @util.only_unix
     def test_sanity_check(self):
         config = pycurl_setup.ExtensionConfiguration()
         # we should link against libcurl, one would expect
         assert 'curl' in config.libraries
 
+    @util.only_unix
     @using_curl_config('curl-config-empty')
     def test_no_ssl(self):
         config = pycurl_setup.ExtensionConfiguration()
         # do not expect anything to do with ssl
         assert 'crypto' not in config.libraries
 
+    @util.only_unix
     @using_curl_config('curl-config-libs-and-static-libs')
     def test_does_not_use_static_libs(self):
         config = pycurl_setup.ExtensionConfiguration()
@@ -77,43 +84,63 @@ class SetupTest(unittest.TestCase):
         assert 'flurby' in config.libraries
         assert 'kzzert' not in config.libraries
 
+    @util.only_unix
     @using_curl_config('curl-config-ssl-in-libs')
     def test_ssl_in_libs(self):
         config = pycurl_setup.ExtensionConfiguration()
         # should link against openssl
         assert 'crypto' in config.libraries
 
+    @util.only_unix
     @using_curl_config('curl-config-ssl-in-static-libs')
     def test_ssl_in_static_libs(self):
         config = pycurl_setup.ExtensionConfiguration()
         # should link against openssl
         assert 'crypto' in config.libraries
 
+    @util.only_unix
     @using_curl_config('curl-config-empty')
     def test_no_ssl_define(self):
         config = pycurl_setup.ExtensionConfiguration()
         # ssl define should be off
         assert 'HAVE_CURL_SSL' not in config.define_symbols
 
+    @util.only_unix
     @using_curl_config('curl-config-ssl-in-libs')
     def test_ssl_in_libs_sets_ssl_define(self):
         config = pycurl_setup.ExtensionConfiguration()
         # ssl define should be on
         assert 'HAVE_CURL_SSL' in config.define_symbols
 
+    @util.only_unix
     @using_curl_config('curl-config-ssl-in-static-libs')
     def test_ssl_in_static_libs_sets_ssl_define(self):
         config = pycurl_setup.ExtensionConfiguration()
         # ssl define should be on
         assert 'HAVE_CURL_SSL' in config.define_symbols
 
-    @using_curl_config('curl-config-ssl-feature-only')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-in-libs')
     def test_ssl_feature_sets_ssl_define(self):
         config = pycurl_setup.ExtensionConfiguration()
         # ssl define should be on
         assert 'HAVE_CURL_SSL' in config.define_symbols
 
-    @using_curl_config('curl-config-empty')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
+    def test_ssl_feature_only(self):
+        try:
+            pycurl_setup.ExtensionConfiguration()
+        except pycurl_setup.ConfigurationError as e:
+            self.assertEqual('''\
+Curl is configured to use SSL, but we have not been able to determine \
+which SSL backend it is using. Please see PycURL documentation for how to \
+specify the SSL backend manually.''', str(e))
+        else:
+            self.fail('Should have raised')
+
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
     def test_libcurl_ssl_openssl(self):
         config = pycurl_setup.ExtensionConfiguration(['',
             '--libcurl-dll=tests/fake-curl/libcurl/with_openssl.so'])
@@ -125,7 +152,8 @@ class SetupTest(unittest.TestCase):
         assert 'HAVE_CURL_GNUTLS' not in config.define_symbols
         assert 'HAVE_CURL_NSS' not in config.define_symbols
 
-    @using_curl_config('curl-config-empty')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
     def test_libcurl_ssl_gnutls(self):
         config = pycurl_setup.ExtensionConfiguration(['',
             '--libcurl-dll=tests/fake-curl/libcurl/with_gnutls.so'])
@@ -137,7 +165,8 @@ class SetupTest(unittest.TestCase):
         assert 'HAVE_CURL_OPENSSL' not in config.define_symbols
         assert 'HAVE_CURL_NSS' not in config.define_symbols
 
-    @using_curl_config('curl-config-empty')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
     def test_libcurl_ssl_nss(self):
         config = pycurl_setup.ExtensionConfiguration(['',
             '--libcurl-dll=tests/fake-curl/libcurl/with_nss.so'])
@@ -149,6 +178,7 @@ class SetupTest(unittest.TestCase):
         assert 'HAVE_CURL_OPENSSL' not in config.define_symbols
         assert 'HAVE_CURL_GNUTLS' not in config.define_symbols
 
+    @util.only_unix
     @using_curl_config('curl-config-empty')
     def test_libcurl_ssl_unrecognized(self):
         config = pycurl_setup.ExtensionConfiguration(['',
@@ -158,7 +188,8 @@ class SetupTest(unittest.TestCase):
         assert 'HAVE_CURL_GNUTLS' not in config.define_symbols
         assert 'HAVE_CURL_NSS' not in config.define_symbols
 
-    @using_curl_config('curl-config-empty')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
     def test_with_ssl_library(self):
         config = pycurl_setup.ExtensionConfiguration(['',
             '--with-ssl'])
@@ -169,7 +200,8 @@ class SetupTest(unittest.TestCase):
         assert 'HAVE_CURL_GNUTLS' not in config.define_symbols
         assert 'HAVE_CURL_NSS' not in config.define_symbols
 
-    @using_curl_config('curl-config-empty')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
     def test_with_openssl_library(self):
         config = pycurl_setup.ExtensionConfiguration(['',
             '--with-openssl'])
@@ -180,7 +212,8 @@ class SetupTest(unittest.TestCase):
         assert 'HAVE_CURL_GNUTLS' not in config.define_symbols
         assert 'HAVE_CURL_NSS' not in config.define_symbols
 
-    @using_curl_config('curl-config-empty')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
     def test_with_gnutls_library(self):
         config = pycurl_setup.ExtensionConfiguration(['',
             '--with-gnutls'])
@@ -191,7 +224,8 @@ class SetupTest(unittest.TestCase):
         assert 'HAVE_CURL_OPENSSL' not in config.define_symbols
         assert 'HAVE_CURL_NSS' not in config.define_symbols
 
-    @using_curl_config('curl-config-empty')
+    @util.only_unix
+    @using_curl_config('curl-config-ssl-feature-only')
     def test_with_nss_library(self):
         config = pycurl_setup.ExtensionConfiguration(['',
             '--with-nss'])
@@ -201,3 +235,32 @@ class SetupTest(unittest.TestCase):
 
         assert 'HAVE_CURL_OPENSSL' not in config.define_symbols
         assert 'HAVE_CURL_GNUTLS' not in config.define_symbols
+
+    @util.only_unix
+    @using_curl_config('curl-config-empty')
+    def test_no_ssl_feature_with_libcurl_dll(self):
+        config = pycurl_setup.ExtensionConfiguration(['',
+            '--libcurl-dll=tests/fake-curl/libcurl/with_openssl.so'])
+        # openssl should not be detected
+        assert 'HAVE_CURL_SSL' not in config.define_symbols
+        assert 'HAVE_CURL_OPENSSL' not in config.define_symbols
+        assert 'crypto' not in config.libraries
+
+    @util.only_unix
+    @using_curl_config('curl-config-empty')
+    def test_no_ssl_feature_with_ssl(self):
+        old_stderr = sys.stderr
+        sys.stderr = captured_stderr = StringIO()
+        
+        try:
+            config = pycurl_setup.ExtensionConfiguration(['',
+                '--with-ssl'])
+            # openssl should not be detected
+            assert 'HAVE_CURL_SSL' not in config.define_symbols
+            assert 'HAVE_CURL_OPENSSL' not in config.define_symbols
+            assert 'crypto' not in config.libraries
+        finally:
+            sys.stderr = old_stderr
+        
+        self.assertEqual("Warning: SSL backend specified manually but libcurl does not use SSL",
+            captured_stderr.getvalue().strip())
